--[[

Function for loading and saving settings and data files.

]]--

maxServerID = 0;

-- load window size/position settings from Character-level file
function _G.LoadSettings()
	_G.settings = DecodeNumbers(Turbine.PluginData.Load(Turbine.DataScope.Character, "KillTracker"));
	
	local upToDate = true;

   -- check that settings file is valid and not for an old version of the plugin
	if ((type(settings) ~= "table") or (settings.versionNo == nil) or (string.gsub(settings.versionNo, "%.", "") < string.gsub(versionNo, "%.", ""))) then
		upToDate = false;
		
      if (type(settings) ~= "table") then
			_G.settings = {};
		end
		
      -- if the settings are too out-dated, put in new defaults
		if ((settings.versionNo == nil) or (string.gsub(settings.versionNo, "%.", "") < "090")) then
         _G.settings = {};
			settings.leftPos = (Turbine.UI.Display:GetWidth() - minWidth) / 2;
         settings.topPos = 0;
         settings.width = minWidth;
         settings.height = minHeight;
			settings.iconLeft = 0;
         settings.iconTop = Turbine.UI.Display:GetHeight() - 32;
         settings.iconLocked = false;
         settings.defaultSort = "KillTime";
         settings.defaultSortOrd = "";
         settings.defaultAllChars = false;
      elseif (string.gsub(settings.versionNo, "%.", "") < "094") then
         settings.iconLocked = false;
         settings.defaultSort = "Enemy";
         settings.defaultSortOrd = "DESC";
         settings.defaultAllChars = false;
		end
		
      -- update version number to current
		settings.versionNo = versionNo;
	end

   -- ensure saved settings for width and height are at least at the set minimums
   if (settings.width < minWidth) then
      upToDate = false;
      
      settings.width = minWidth;
   end

   if (settings.height < minHeight) then
      upToDate = false;
      
      settings.height = minHeight;
	end

	-- account for changed resolution
	if (settings.width > Turbine.UI.Display.GetWidth()) then
		settings.width = Turbine.UI.Display.GetWidth();
	end

	if (settings.leftPos < 0) then
		settings.leftPos = 0;
	end
	
	if (settings.leftPos > Turbine.UI.Display.GetWidth() - settings.width) then
		settings.leftPos = Turbine.UI.Display.GetWidth() - settings.width;
	end

	if (settings.height > Turbine.UI.Display.GetHeight()) then
		settings.height = Turbine.UI.Display.GetHeight();
	end

	if (settings.topPos < 0) then
		settings.topPos = 0;
	end

	if (settings.topPos > Turbine.UI.Display.GetHeight() - settings.height) then
		settings.topPos = Turbine.UI.Display.GetHeight() - settings.height;
	end

	if (settings.iconLeft < 0) then
		settings.iconLeft = 0;
	end
	
	if (settings.iconLeft > Turbine.UI.Display.GetWidth() - 32) then
		settings.iconLeft = Turbine.UI.Display.GetWidth() - 32;
	end

	if (settings.iconTop < 0) then
		settings.iconTop = 0;
	end

	if (settings.iconTop > Turbine.UI.Display.GetHeight() - 32) then
		settings.iconTop = Turbine.UI.Display.GetHeight() - 32;
	end
   
	-- save adjusted settings
   SaveSettings();
end

-- save window size/position settings to Character-level file
function _G.SaveSettings()
	Turbine.PluginData.Save(Turbine.DataScope.Character, "KillTracker", EncodeNumbers(settings));
end

-- load main kill log data and ServerID counter from Account-level file
function _G.LoadData()
	local tmpData = DecodeNumbers(Turbine.PluginData.Load(Turbine.DataScope.Account, "KillTracker"));
   
   if (tmpData ~= nil) then
      -- data file corrupt, disable saving so it can be fixed manually, hopefully without data loss
      if (type(tmpData) ~= "table") then
         _G.saveData = false;

         LogMessage("KillTracker: "..L.LoadFailure, 0, true)

			statsWindow.dataFailure:SetVisible(true);
			
         _G.killsTable = {};
      else
         if (type(tmpData[2]) == "table") then
         	_G.killsTable = tmpData[2];
         else
         	_G.killsTable = DecodeKillData(tmpData[2]);
         end
         
			-- for old data files:
			--    -default most recent kill timestamps to the current time minus an offset to keep them in order
			--    -strip any tabs from the serverIDs
         if (#killsTable > 0) then
				if (killsTable[1][6] == nil) then
					tmpTime = Turbine.Engine.GetLocalTime();
														
					for i = 1, #killsTable do
						killsTable[i][5] = string.gsub(killsTable[i][5], "[\t\n]", "");
						killsTable[i][6] = tmpTime - #killsTable + i;
					end
				end

				-- sort by timestamp, newest at the bottom
				table.sort(killsTable, function(arg1, arg2)
						if (arg1[6] < arg2[6]) then
							return(true);
						elseif ((arg1[6] == arg2[6]) and (arg1[1] < arg2[1])) then
							return(true);
						elseif ((arg1[6] == arg2[6]) and (arg1[1] == arg2[1]) and (arg1[5] < arg2[5])) then
							return(true);
						end
					end);
			end
      end
   -- data file not found, init
   else
      LogMessage("KillTracker: Data file not found.", 1, false);

      _G.killsTable = {};
   end
end

-- save main kill log data to Account-level file
function _G.SaveData()
   -- don't save if data file load failed
   if (saveData) then
      local tmpData = {};

		-- skipping index 1 to avoid problems with old save file format
      tmpData[2] = killsTable;

      Turbine.PluginData.Save(Turbine.DataScope.Account, "KillTracker", EncodeNumbers(tmpData));
   end
end

-- load Server labels from Account-level file
function _G.LoadServers()
	local tmpData = DecodeNumbers(Turbine.PluginData.Load(Turbine.DataScope.Account, "KillTrackerServers"));
   
   if (tmpData ~= nil) then
      -- data file corrupt, disable saving so it can be fixed manually, hopefully without data loss
      if (type(tmpData) ~= "table") then
         _G.saveData = false;

         LogMessage("KillTracker: "..L.LoadFailure, 0, true)

			statsWindow.dataFailure:SetVisible(true);
			
         _G.serverLabels = {};
      else
         maxServerID = tmpData[1];

			if (tmpData[2] == nil) then
         	_G.serverLabels = {};
         else
         	_G.serverLabels = tmpData[2];
         end
      end
   -- data file not found, init
   else
      LogMessage("KillTracker: Server data file not found.", 1, false);

		_G.serverLabels = {};
   end
end

-- save Server Labels to Account-level file
function _G.SaveServers()
   -- don't save if data file load failed
   if (saveData) then
      local tmpData = {};

      tmpData[1] = maxServerID;
      tmpData[2] = serverLabels;

      Turbine.PluginData.Save(Turbine.DataScope.Account, "KillTrackerServers", EncodeNumbers(tmpData));
   end
end

-- load/set/save Server ID for current server from/to Server-level file
function _G.LoadServerID()
	_G.serverID = DecodeNumbers(Turbine.PluginData.Load(Turbine.DataScope.Server, "KillTracker"));

	-- no Server ID set, set one (won't be any kill data)
   if (serverID == nil) then
      -- don't save if data file load failed
      if (saveData) then
			-- increment max Server ID from Account-level file
			maxServerID = maxServerID + 1;
			
			-- set new Server ID
			_G.serverID = maxServerID;
			
			serverLabels[serverID] = ""..GetNewServerName();  
         
         -- write new max Server ID and labels to Account-level file first to ensure no re-use
         SaveServers();

			-- save new Server ID      
         SaveServerID();
      -- data file load failed, system in view-only mode, Server ID doesn't matter
      else
         _G.serverID = 999999;

         serverLabels[serverID] = "***"; 
      end
	-- server ID was a string, set new numeric and store label (might be kill data, but only if the name is still unique)
	elseif (type(serverID) == "string") then
		local oldServerID = string.gsub(serverID, "[\(\)\*\t\n]", "");
		local newName = oldServerID;

		-- increment max Server ID from Account-level file
		maxServerID = maxServerID + 1;
		
		-- set new Server ID
		_G.serverID = maxServerID;

		local isNameOK = true;	

		-- check if server name already in use
		for i = 1, #serverLabels do
			if (serverLabels[i] == oldServerID) then
				isNameOK = false;
				
				break;
			end
		end

		if (not isNameOK) then
			newName = GetNewServerName();
		end
		
		serverLabels[serverID] = ""..newName;  
		
		-- write new max Server ID and labels to Account-level file first to ensure no re-use
		SaveServers();
		
		-- save new Server ID      
		SaveServerID();
	-- server label is missing, set error state
	elseif (serverLabels[serverID] == nil) then
		_G.saveData = false;
		
		LogMessage("KillTracker: "..L.LoadFailure, 0, true)
		
		statsWindow.dataFailure:SetVisible(true);

		_G.serverID = 999999;
		
		serverLabels[serverID] = "***"; 
	end
   
   -- set character unique name as CharacterName(ServerID), used to distinguish
   --   same-named characters on different servers within same account
   player.uniqueName = player.name.."("..serverLabels[serverID]..")";
end

-- write new Server ID to Server-level file
function _G.SaveServerID()
   -- don't save if data file load failed
   if (saveData) then
		Turbine.PluginData.Save(Turbine.DataScope.Server, "KillTracker", EncodeNumbers(serverID));
	end
end

function DecodeKillData(killData)
	local tmpKillTable = {};
	
	if (killData == nil) then
		return tmpKillTable;
	elseif (killData == "") then
		return tmpKillTable;
	else
		local rowIndex = 1; 

		for rowStr in string.gmatch(killData.."\n", "(.-)\n") do
			tmpKillTable[rowIndex] = {};

			local colIndex = 1;
			
			for colStr in string.gmatch(rowStr.."\t", "(.-)\t") do 
				if ((colIndex == 2) or (colIndex == 3) or (colIndex == 4) or (colIndex == 6)) then
					tmpKillTable[rowIndex][colIndex] = tonumber(colStr);
				else
					tmpKillTable[rowIndex][colIndex] = colStr;
				end
				
				colIndex = colIndex + 1;
			end
			
			rowIndex = rowIndex + 1;
		end
		
		return tmpKillTable;
	end	
end

function GetNewServerName()
	tmpServerID = 1;

	repeat
		local isNameOK = true;

		-- server name already in use, reset
		for i = 1, #serverLabels do
			if (serverLabels[i] == ""..tmpServerID) then
				isNameOK = false;

				tmpServerID = tmpServerID + 1;
				
				break;
			end
		end
	until (isNameOK)
	
	return tmpServerID;
end