--[[

A generic timer object than only runs once per second.

]]--

_G.Timer = class(Turbine.UI.Control);

function Timer:Constructor()
	Turbine.UI.Control.Constructor(self);
	
	self.expiresIn = 0;
	self.lastRunSec = 0;
	self:SetVisible(false);
end

function Timer:Start()
	self.lastRunSec = Turbine.Engine.GetLocalTime();
	self:SetWantsUpdates(true);
end

function Timer:Stop()
	self:SetWantsUpdates(false);
end

function Timer:Update()
	local timestamp = Turbine.Engine.GetLocalTime();

	if (self.lastRunSec < timestamp) then
		self.expiresIn = self.expiresIn - (timestamp - self.lastRunSec);

		self.lastRunSec = timestamp;

		if (type(self.OnUpdate) == "function") then
			self:OnUpdate(timestamp);
		end
	
		if (self.expiresIn <= 0) then
			self:SetWantsUpdates(false);
	
			if (type(self.OnExpiration) == "function") then
				self:OnExpiration();
			end
		end
	end
end
